#--------------------------------------------------
# EXAMINE FIT OF IFM ROBUST USING SIMULATED DATA
#--------------------------------------------------
load("SIMULATE_DATA_100_20_10.RData")

#NOTE: YOU WILL NEED TO RENAME SUPPLEMENTARY FILES IN ORDER TO LOAD FUNCTIONS.
#FILE NAMES FOR SOURCE FUNCTIONS APPEAR IN COMMENTS AT THE BEGINNING OF THE
#RESPECTIVE FILES
source("1_MCMC_FUNCTIONS.R")
source("3_IFM_ROBUST_MCMC_FUNCTION.R")
source("5_CHAIN_DIAGNOSTICS_FUNCTIONS.R")

#THESE PARAMETERS WILL NEED TO BE EDITED TO CORRESPOND TO THE SIMULATED DATA:
niter=5000
nsite=100
nyear=20
nthin=1
nrep=10
seed=101

#TRUE PARAMETER ESTIMATES:
# MUPSI1:
mean(z.sim.no.missing.values) 
# ALPHA: 0.2
# BETA: 0.5
# GAMMA: 10
# MU: 0.25
# CHI: 0.25
# p=rep(c(0.4,0.6,0.2,0.5,0.3),2)
z=rep(0,nsite*nyear)
dim(z)=c(nsite,nyear)
for(i in 1:nyear) {
	z[,i]=apply(sim.det.no.missing.values[,i,],1,max,na.rm=TRUE)
	}
z[z==-Inf]=NA
nmissing=sum(is.na(z))

sim.init1=list(z.data=z,z.missing=runif(nmissing),p=runif(nyear),mupsi1=runif(1),log.alpha=runif(1,-2,0.5), log.beta=runif(1,-2,0.5),log.gamma=runif(1,-2,0.5),log.mu=runif(1,-2,0.5),log.chi=runif(1,-2,0.5))

sim.robust1=ifm.robust.MCMC(niter=niter,init=sim.init1, det.data=sim.det.no.missing.values, sim.distance=sim.distance,sim.area=sim.area, sd.prop.alpha=0.2,sd.prop.beta=0.75,sd.prop.gamma=0.3,sd.prop.mu=0.3,sd.prop.chi=0.75,gamma.beta.bivnorm.high=FALSE,gamma.beta.bivnorm.low=FALSE,nsite.subset=10,nthin=nthin,print.by=100,seed=seed)

accept.calculate(option=3,x=sim.robust1)

sim.init2=list(z.data=z,z.missing=runif(nmissing),p=runif(nyear),mupsi1=runif(1),log.alpha=runif(1,-2,0.5), log.beta=runif(1,-2,0.5),log.gamma=runif(1,-2,0.5),log.mu=runif(1,-2,0.5),log.chi=runif(1,-2,0.5))

sim.robust2=ifm.robust.MCMC(niter=niter,init=sim.init2, det.data=sim.det.no.missing.values, sim.distance=sim.distance,sim.area=sim.area, sd.prop.alpha=0.2,sd.prop.beta=0.75,sd.prop.gamma=0.3,sd.prop.mu=0.3,sd.prop.chi=0.75,gamma.beta.bivnorm.high=FALSE,gamma.beta.bivnorm.low=FALSE,nsite.subset=10,nthin=nthin,print.by=100)

sim.robust1=p.list.insert(sim.robust1)
sim.robust2=p.list.insert(sim.robust2)

coda.create(sim.robust1,"sim_robust1",par.list=list("mupsi1.chain","log.mu.chain","log.chi.chain","log.beta.chain","log.gamma.chain","log.alpha.chain","p.1","p.2","p.3","p.4","p.5","p.6","p.7","p.8","p.9","p.10"),niter=niter,nthin=nthin)
coda.create(sim.robust2,"sim_robust2",par.list=list("mupsi1.chain","log.mu.chain","log.chi.chain","log.beta.chain","log.gamma.chain","log.alpha.chain","p.1","p.2","p.3","p.4","p.5","p.6","p.7","p.8","p.9","p.10"),niter=niter,nthin=nthin)

sim.temp1=read.coda("sim_robust1.txt","sim_robust1_Index.txt")
sim.temp2=read.coda("sim_robust2.txt","sim_robust2_Index.txt")
sim.temp.list=mcmc.list(sim.temp1,sim.temp2)

pdf("temp2.pdf")
plot(sim.temp.list)
dev.off()


# MUPSI1: 
mean(z.sim.no.missing.values[,1]) 

summary(sim.temp.list)
apply(z.sim.no.missing.values,2,mean)
apply(sim.robust1$muz.chain,1,mean)

save(sim.temp.list,file="AUDIT_SIM_100_10_10_b0pt5.RData")
